/*

RC6-512 edited by Alexander Myasnikov

Web: www.darksoftware.narod.ru

*/


#include <stdlib.h>

typedef unsigned long u32;
typedef unsigned char u8;
typedef u32 rc6_key[44];


#define num_rounds 20
#define keylen 64

__inline u32
BSWAP (u32 x)
{
  __asm
  {
    mov eax, x;
    bswap eax;
  }
  return x;
}


#define ROL(x, y) _lrotl(x, y)
#define ROR(x, y) _lrotr(x, y)
#define MAX(x, y) ( ((x)>(y))?(x):(y) )


rc6_key skey;

void __stdcall __export
setup (const u8 * key)
{
  u32 L[64], S[50], A, B, i, j, v, s, t, l;



  /* copy the key into the L array */
  for (A = i = j = 0; i < keylen;)
    {
      A = (A << 8) | ((u32) (key[i++] & 255));
      if (!(i & 3))
	{
	  L[j++] = BSWAP (A);
	  A = 0;
	}
    }


  /* setup the S array */
  t = 44;			/* fixed at 20 rounds */
  S[0] = 0xB7E15163UL;
  for (i = 1; i < t; i++)
    S[i] = S[i - 1] + 0x9E3779B9UL;

  /* mix buffer */
  s = 3 * MAX (t, j);
  l = j;
  for (A = B = i = j = v = 0; v < s; v++)
    {
      A = S[i] = ROL (S[i] + A + B, 3);
      B = L[j] = ROL (L[j] + A + B, (A + B));
      i = (i + 1) % t;
      j = (j + 1) % l;
    }

  /* copy to key */
  for (i = 0; i < t; i++)
    {
      skey[i] = S[i];
    }

}


void __stdcall __export
crypt (u32 * pt)
{
  u32 a, b, c, d, t, u;
  int r;

  a = pt[0];
  b = pt[1];
  c = pt[2];
  d = pt[3];

  b += skey[0];
  d += skey[1];
  for (r = 0; r < 20; r++)
    {
      t = (b * (b + b + 1));
      t = ROL (t, 5);
      u = (d * (d + d + 1));
      u = ROL (u, 5);
      a = ROL (a ^ t, u) + skey[r + r + 2];
      c = ROL (c ^ u, t) + skey[r + r + 3];
      t = a;
      a = b;
      b = c;
      c = d;
      d = t;
    }
  a += skey[42];
  c += skey[43];
  pt[0] = a;
  pt[1] = b;
  pt[2] = c;
  pt[3] = d;

}

void __stdcall __export
decrypt (u32 * ct)
{
  u32 a, b, c, d, t, u;
  int r;

  a = ct[0];
  b = ct[1];
  c = ct[2];
  d = ct[3];

  a -= skey[42];
  c -= skey[43];
  for (r = 19; r >= 0; r--)
    {
      t = d;
      d = c;
      c = b;
      b = a;
      a = t;
      t = (b * (b + b + 1));
      t = ROL (t, 5);
      u = (d * (d + d + 1));
      u = ROL (u, 5);
      c = ROR (c - skey[r + r + 3], t) ^ u;
      a = ROR (a - skey[r + r + 2], u) ^ t;
    }
  b -= skey[0];
  d -= skey[1];

  ct[0] = a;
  ct[1] = b;
  ct[2] = c;
  ct[3] = d;
}
